#!/usr/bin/env python3

import os
import sys
import time
import json
import logging
from pathlib import Path
from typing import Set, List, Tuple
import argparse
from datetime import datetime

import requests
from bs4 import BeautifulSoup, NavigableString, Comment
import deepl

logging.basicConfig(
    level=logging.INFO,
    format='%(asctime)s - %(levelname)s - %(message)s',
    handlers=[
        logging.StreamHandler(sys.stdout),
        logging.FileHandler('translation.log', encoding='utf-8')
    ]
)
logger = logging.getLogger(__name__)

class HTMLTranslator:
    def __init__(self, api_key: str, target_lang: str, source_lang: str = 'en'):

        self.translator = deepl.Translator(api_key)
        self.target_lang = target_lang.upper()
        self.source_lang = source_lang.upper()
        self.registry_file = "registry.txt"
        self.translated_files: Set[str] = set()

        self.skip_tags = {
            'script', 'style', 'code', 'pre', 'noscript', 'iframe',
            'object', 'embed', 'video', 'audio', 'canvas', 'svg'
        }

        self.translatable_attrs = {
            'title', 'alt', 'placeholder', 'aria-label', 'aria-description'
        }

        self._load_registry()
        self._validate_languages()

    def _validate_languages(self):

        try:

            source_langs = self.translator.get_source_languages()
            target_langs = self.translator.get_target_languages()

            source_codes = [lang.code for lang in source_langs]
            target_codes = [lang.code for lang in target_langs]

            if self.source_lang not in source_codes:
                raise ValueError(f"Source language '{self.source_lang}' not supported")

            if self.target_lang not in target_codes:
                raise ValueError(f"Target language '{self.target_lang}' not supported")

            logger.info(f"Translating from {self.source_lang} to {self.target_lang}")

        except Exception as e:
            logger.error(f"Error validating languages: {e}")
            sys.exit(1)

    def _load_registry(self):
        if os.path.exists(self.registry_file):
            try:
                with open(self.registry_file, 'r', encoding='utf-8') as f:
                    for line in f:
                        line = line.strip()
                        if line:
                            self.translated_files.add(line)
                logger.info(f"Loaded {len(self.translated_files)} files from registry")
            except Exception as e:
                logger.error(f"Error loading registry: {e}")

    def _save_to_registry(self, file_path: str):
        try:
            with open(self.registry_file, 'a', encoding='utf-8') as f:
                f.write(f"{file_path}\n")
            self.translated_files.add(file_path)
        except Exception as e:
            logger.error(f"Error saving to registry: {e}")

    def _find_html_files(self, directory: str) -> List[str]:
        html_files = []
        for root, dirs, files in os.walk(directory):
            for file in files:
                if file.lower().endswith(('.html', '.htm')):
                    full_path = os.path.join(root, file)
                    html_files.append(full_path)
        return html_files

    def _translate_text(self, text: str) -> str:
        if not text or not text.strip():
            return text

        try:
            result = self.translator.translate_text(
                text,
                source_lang=self.source_lang,
                target_lang=self.target_lang
            )
            return result.text
        except Exception as e:
            logger.error(f"Error translating '{text[:50]}...': {e}")
            return text

    def _should_translate_tag(self, tag) -> bool:
        if tag.name in self.skip_tags:
            return False

        if tag.get('class'):
            classes = ' '.join(tag['class']).lower()
            if any(keyword in classes for keyword in ['code', 'highlight', 'syntax']):
                return False

        return True

    def _translate_element(self, element):
        if isinstance(element, Comment):
            return

        if isinstance(element, NavigableString):
            text = str(element).strip()
            if text and any(c.isalpha() for c in text):
                translated = self._translate_text(text)
                if translated != text:
                    element.replace_with(translated)
            return

        if not self._should_translate_tag(element):
            return

        for attr in self.translatable_attrs:
            if element.get(attr):
                original_attr = element[attr]
                if isinstance(original_attr, str) and original_attr.strip():
                    translated_attr = self._translate_text(original_attr)
                    element[attr] = translated_attr

        for child in list(element.children):
            self._translate_element(child)

    def _translate_html_file(self, file_path: str) -> bool:
        try:
            with open(file_path, 'r', encoding='utf-8') as f:
                content = f.read()

            soup = BeautifulSoup(content, 'html.parser')

            body = soup.find('body')
            if body:
                self._translate_element(body)
            else:
                for element in soup.find_all():
                    self._translate_element(element)

            title = soup.find('title')
            if title and title.string:
                title.string = self._translate_text(title.string)

            meta_desc = soup.find('meta', attrs={'name': 'description'})
            if meta_desc and meta_desc.get('content'):
                meta_desc['content'] = self._translate_text(meta_desc['content'])

            with open(file_path, 'w', encoding='utf-8') as f:
                f.write(str(soup))

            return True

        except Exception as e:
            logger.error(f"Error translating {file_path}: {e}")
            return False

    def translate_directory(self, directory: str):
        if not os.path.exists(directory):
            logger.error(f"Directory '{directory}' not found")
            return

        logger.info(f"Starting translation of directory: {directory}")
        logger.info(f"Target language: {self.target_lang}")

        html_files = self._find_html_files(directory)
        total_files = len(html_files)

        if total_files == 0:
            logger.info("No HTML files found")
            return

        logger.info(f"Found {total_files} HTML files")

        files_to_translate = [f for f in html_files if f not in self.translated_files]
        remaining_files = len(files_to_translate)

        logger.info(f"Already translated files: {total_files - remaining_files}")
        logger.info(f"Files to translate: {remaining_files}")

        if remaining_files == 0:
            logger.info("All files have already been translated!")
            return

        start_time = datetime.now()

        for i, file_path in enumerate(files_to_translate, 1):
            logger.info(f"[{i}/{remaining_files}] Translating: {file_path}")

            if self._translate_html_file(file_path):
                self._save_to_registry(file_path)
                logger.info(f"✓ Successfully translated: {file_path}")
            else:
                logger.error(f"✗ Error translating: {file_path}")

            remaining = remaining_files - i
            logger.info(f"Progress: {i}/{remaining_files} - Remaining: {remaining}")

            time.sleep(0.5)

        end_time = datetime.now()
        duration = end_time - start_time

        logger.info(f"Translation completed in {duration}")
        logger.info(f"Files translated: {remaining_files}")

def main():
    parser = argparse.ArgumentParser(description="Recursive HTML Translator with DeepL")
    parser.add_argument("directory", help="Directory containing HTML files")
    parser.add_argument("--api-key", required=True, help="DeepL API key")
    parser.add_argument("--target-lang", required=True,
                       help="Target language (it, es, de, fr, etc.)")
    parser.add_argument("--source-lang", default="en",
                       help="Source language (default: en)")

    args = parser.parse_args()

    if not os.path.exists(args.directory):
        logger.error(f"Directory '{args.directory}' not found")
        sys.exit(1)

    try:

        translator = HTMLTranslator(
            api_key=args.api_key,
            target_lang=args.target_lang,
            source_lang=args.source_lang
        )

        translator.translate_directory(args.directory)

    except KeyboardInterrupt:
        logger.info("\nTranslation interrupted by user")
        logger.info("Progress has been saved in registry.txt")
    except Exception as e:
        logger.error(f"Fatal error: {e}")
        sys.exit(1)

if __name__ == "__main__":
    main()
