import os
import sys
from bs4 import BeautifulSoup, NavigableString, Comment
from deep_translator import GoogleTranslator
from concurrent.futures import ThreadPoolExecutor
import time

#Put the index in a separate folder and run the script, unfortunately the index is bugged and cannot be translated with the normal script, for change language change target="it" to your lang

base_dir = os.path.abspath(os.path.dirname(__file__))
registry_file = os.path.join(base_dir, 'registry.txt')

SKIP_TAGS = ['script', 'style', 'meta', 'link', 'head', 'noscript']

def load_registry():
    if os.path.exists(registry_file):
        with open(registry_file, 'r', encoding='utf-8') as f:
            return set(line.strip() for line in f if line.strip())
    return set()

def save_to_registry(path):
    with open(registry_file, 'a', encoding='utf-8') as f:
        f.write(f"{path}\n")

def translate_text(text, target='it'):
    text = text.strip()
    if len(text) < 2:
        return text
    try:
        translated = GoogleTranslator(source='en', target=target).translate(text)
        return translated if translated else text
    except Exception as e:
        print(f"⚠️ Translation error: '{text[:30]}...': {e}")
        return text

def translate_batch_texts(texts, target='it'):
    try:
        return GoogleTranslator(source='en', target=target).translate_batch(texts)
    except Exception as e:
        print(f"⚠️ Batch translation error: {e}")
        return texts

def extract_text_nodes(soup):
    nodes = []
    def recurse(node):
        if isinstance(node, NavigableString) and not isinstance(node, Comment):
            text = str(node).strip()
            if text:
                nodes.append(node)
        elif hasattr(node, 'contents') and node.name not in SKIP_TAGS:
            for child in node.contents:
                recurse(child)
    recurse(soup)
    return nodes

def translate_html(file_path, method='single', target_language='it'):
    try:
        with open(file_path, 'r', encoding='utf-8') as file:
            soup = BeautifulSoup(file, 'html.parser')

        nodes = extract_text_nodes(soup)

        if method == 'batch':
            original_texts = [str(n).strip() for n in nodes]
            translations = translate_batch_texts(original_texts, target_language)
            for node, translated in zip(nodes, translations):
                if translated:
                    node.replace_with(translated)

        else:
            for node in nodes:
                original = str(node).strip()
                if original:
                    translated = translate_text(original, target_language)
                    node.replace_with(translated)

        with open(file_path, 'w', encoding='utf-8') as out_file:
            out_file.write(str(soup))

        save_to_registry(file_path)
        print(f"✅ Translated: {file_path}")
        return True

    except Exception as e:
        print(f"❌ Error with file {file_path}: {e}")
        return False

def get_all_html_files(base_dir, already_done):
    html_files = []
    for root, dirs, files in os.walk(base_dir):
        for filename in files:
            if filename.endswith('.html'):
                file_path = os.path.join(root, filename)
                if file_path not in already_done:
                    html_files.append(file_path)
    return html_files

def scan_and_translate_serial(files):
    total = len(files)
    for idx, file_path in enumerate(files, 1):
        translate_html(file_path, method='single')
        print(f"📂 Translated {idx}/{total} files, {total - idx} remaining...")

def scan_and_translate_parallel(files):
    total = len(files)
    translated = 0
    with ThreadPoolExecutor(max_workers=5) as executor:
        futures = {executor.submit(translate_html, f, 'single'): f for f in files}
        for idx, future in enumerate(futures, 1):
            future.result()
            translated += 1
            print(f"📂 Translated {translated}/{total} files, {total - translated} remaining...")

def scan_and_translate_batch(files):
    total = len(files)
    for idx, file_path in enumerate(files, 1):
        translate_html(file_path, method='batch')
        print(f"📂 Translated {idx}/{total} files, {total - idx} remaining...")

if __name__ == '__main__':
    print("Select translation mode:\n1 = Normal\n2 = Parallel\n3 = Batch")
    method = input("Enter 1, 2 or 3: ").strip()

    if method not in ['1', '2', '3']:
        print("❌ Invalid input. Enter 1, 2 or 3.")
        sys.exit(1)

    done = load_registry()
    files_to_translate = get_all_html_files(base_dir, done)

    print(f"📊 Remaining files to translate: {len(files_to_translate)}")

    if method == '1':
        scan_and_translate_serial(files_to_translate)
    elif method == '2':
        scan_and_translate_parallel(files_to_translate)
    elif method == '3':
        scan_and_translate_batch(files_to_translate)

    print("\n✅ Translation completed.")
