import os
import sys
import time
import platform
from bs4 import BeautifulSoup, NavigableString
import deepl
from concurrent.futures import ThreadPoolExecutor, as_completed
import re
import subprocess
from datetime import datetime

base_dir = os.path.abspath(os.path.dirname(__file__))
registry_file = os.path.join(base_dir, "registry.txt")
log_file_path = os.path.join(base_dir, "translation_report.log")

success_count = 0
error_count = 0
skipped_count = 0
translated_files_list = []
error_files_list = []

def generate_requirements_txt():
    requirements = [
        "beautifulsoup4",
        "deepl"
    ]
    with open("requirements.txt", "w") as f:
        f.write("\n".join(requirements))
    print("📦 requirements.txt file created.")

def install_dependencies():
    os_name = platform.system().lower()
    print(f"💻 Detected operating system: {os_name}")
    cmd = [sys.executable, "-m", "pip", "install", "-r", "requirements.txt"]
    try:
        subprocess.check_call(cmd)
        print("✅ Dependencies installed successfully.")
    except subprocess.CalledProcessError as e:
        print(f"❌ Error during installation: {e}")

def get_deepl_translator():
    api_key = os.getenv("DEEPL_API_KEY")
    if not api_key:
        raise ValueError("❌ DeepL API key not found. Please set the DEEPL_API_KEY environment variable.")
    return deepl.Translator(api_key)

def load_registry():
    if not os.path.exists(registry_file):
        return set()
    with open(registry_file, 'r', encoding='utf-8') as f:
        return set(line.strip() for line in f if line.strip())

def update_registry(file_path):
    with open(registry_file, 'a', encoding='utf-8') as f:
        f.write(file_path + '\n')

def translate_text(text, target='it'):
    text = text.strip()
    if len(text) < 2:
        return text
    try:
        translator = get_deepl_translator()
        pattern = re.compile(r'([^\w\s\.,;:!?\'"\-_ \(\)\[\]\{\}<>@#$%&*+=\/|\\~`]+)')
        parts = pattern.split(text)
        for i in range(0, len(parts), 2):
            if parts[i]:
                parts[i] = translator.translate_text(parts[i], target_lang=target.upper()).text
        return ''.join(parts)
    except Exception as e:
        print(f"⚠️ Error translating: '{text[:30]}...': {e}")
        return text

def batch_translate_texts(texts, target='it'):
    try:
        translator = get_deepl_translator()
        all_parts = []
        part_indices = []
        pattern = re.compile(r'([^\w\s\.,;:!?\'"\-_\(\)\[\]\{\}<>@#$%&*+=\/|\\~`]+)')

        for text_idx, text in enumerate(texts):
            parts = pattern.split(text or '')
            for i in range(0, len(parts), 2):
                if parts[i]:
                    all_parts.append(parts[i])
                    part_indices.append((text_idx, i))

        if all_parts:
            translated_parts = translator.translate_text(all_parts, target_lang=target.upper())
            result_texts = [list(pattern.split(text or '')) for text in texts]

            for (text_idx, part_idx), translated in zip(part_indices, translated_parts):
                result_texts[text_idx][part_idx] = translated.text if translated else ''

            return [''.join(part if part is not None else '' for part in parts) for parts in result_texts]

        return texts

    except Exception as e:
        print(f"⚠️ Error in batch translation: {e}")
        return texts

def translate_node_text_batch(node, target_language='it'):
    if node.name in ['script', 'style']:
        return
    texts_to_translate = []
    text_nodes = []
    for child in node.descendants:
        if isinstance(child, NavigableString):
            original = str(child).strip()
            if original:
                texts_to_translate.append(original)
                text_nodes.append(child)
    if texts_to_translate:
        translated_texts = batch_translate_texts(texts_to_translate, target_language)
        for original_node, translated in zip(text_nodes, translated_texts):
            try:
                if original_node != translated:
                    original_node.replace_with(translated)
            except Exception as e:
                print(f"⚠️ Error replacing '{str(original_node)[:30]}...': {e}")

def fix_doctype(soup):
    doctype = '<!DOCTYPE html>'
    if not str(soup).lower().startswith(doctype.lower()):
        html = str(soup)
        if html.strip().lower().startswith('html'):
            html = html.split('\n', 1)[1]
        return doctype + '\n' + html
    return str(soup)

def update_html_lang(soup, lang_code):
    html_tag = soup.find('html')
    if html_tag and html_tag.has_attr('lang'):
        html_tag['lang'] = lang_code

def translate_html(file_path, mode='standard', target_language='it'):
    global success_count, error_count, skipped_count
    try:
        with open(file_path, 'r', encoding='utf-8') as file:
            soup = BeautifulSoup(file, 'html.parser')

        if mode == 'batch':
            translate_node_text_batch(soup, target_language)
        else:
            translate_node_text_batch(soup, target_language)

        update_html_lang(soup, target_language)
        final_html = fix_doctype(soup)

        with open(file_path, 'w', encoding='utf-8') as out_file:
            out_file.write(final_html)

        update_registry(file_path)
        print(f"✅ Translated: {file_path}")
        success_count += 1
        translated_files_list.append(file_path)
        return True

    except Exception as e:
        print(f"❌ Error with file {file_path}: {e}")
        error_count += 1
        error_files_list.append(file_path)
        return False

def scan_and_translate(current_dir, mode='standard', parallel=False, target_language='it'):
    global success_count, error_count, skipped_count
    translated_files = load_registry()
    html_files = []
    for root, dirs, files in os.walk(current_dir):
        for filename in files:
            if filename.endswith('.html'):
                file_path = os.path.join(root, filename)
                if file_path not in translated_files:
                    html_files.append(file_path)
                else:
                    skipped_count += 1

    total_files = len(html_files)
    print(f"\n📊 Files to translate: {total_files} (skipped {skipped_count})")

    if parallel:
        with ThreadPoolExecutor(max_workers=4) as executor:
            futures = {executor.submit(translate_html, path, mode, target_language): path for path in html_files}
            for idx, future in enumerate(as_completed(futures), start=1):
                future.result()
                print(f"📂 Translated {idx}/{total_files} files, {total_files - idx} remaining...")
    else:
        for idx, file_path in enumerate(html_files, start=1):
            translate_html(file_path, mode, target_language)
            print(f"📂 Translated {idx}/{total_files} files, {total_files - idx} remaining...")

def save_report():
    try:
        with open(log_file_path, 'a', encoding='utf-8') as f:
            f.write(f"\n--- Report generated on {datetime.now().strftime('%Y-%m-%d %H:%M:%S')} ---\n")
            f.write(f"Total translated files: {success_count}\n")
            for fpath in translated_files_list:
                f.write(f"  ✔ {fpath}\n")
            f.write(f"Total error files: {error_count}\n")
            for fpath in error_files_list:
                f.write(f"  ✘ {fpath}\n")
            f.write(f"Total skipped files (already registered): {skipped_count}\n")
            f.write("✅ Translation completed.\n")
    except Exception as e:
        print(f"❌ Error saving report: {e}")

def main():
    print("🌍 Choose the target language (e.g., it, es, fr, de...)")
    target = input("Language code: ").strip().lower()

    print("\nChoose translation mode:")
    print("1. Standard (one request at a time)")
    print("2. Batch (multiple texts together)")
    print("3. Parallel + standard")
    choice = input("Enter 1, 2 or 3: ").strip()

    if choice == '1':
        scan_and_translate(base_dir, mode='standard', parallel=False, target_language=target)
    elif choice == '2':
        scan_and_translate(base_dir, mode='batch', parallel=False, target_language=target)
    elif choice == '3':
        scan_and_translate(base_dir, mode='standard', parallel=True, target_language=target)
    else:
        print("❌ Invalid choice. Try again.")
        sys.exit(1)

    print("\n✅ Translation completed.")

if __name__ == '__main__':
    try:
        if "--setup" in sys.argv:
            generate_requirements_txt()
            install_dependencies()
            print("✅ Setup completed. You can now run the script normally.")
            sys.exit(0)

        main()

    except KeyboardInterrupt:
        print("\n⏹ Process interrupted by user.")
    finally:
        save_report()
        print("✅ Report saved successfully. Clean exit.")
