import os
import sys
import time
import platform
import tempfile
import shutil
from bs4 import BeautifulSoup, NavigableString
from concurrent.futures import ThreadPoolExecutor, as_completed
import re
from datetime import datetime
import fitz
from weasyprint import HTML
import base64
import unicodedata
import deepl

# ================== CONFIG API DEEPL ==================
DEEPL_API_KEY = "INSERT_YOUR_DEEPL_KEY_HERE"
translator_deepl = deepl.Translator(DEEPL_API_KEY)
# ==============================================================

base_dir = os.path.abspath(os.path.dirname(__file__))
registry_file = os.path.join(base_dir, "registry.txt")
log_file_path = os.path.join(base_dir, "translation_report.log")

success_count = 0
error_count = 0
skipped_count = 0
translated_files_list = []
error_files_list = []

def load_registry():
    if not os.path.exists(registry_file):
        return set()
    with open(registry_file, 'r', encoding='utf-8') as f:
        return set(line.strip() for line in f if line.strip())

def update_registry(file_path):
    with open(registry_file, 'a', encoding='utf-8') as f:
        f.write(file_path + '\n')

def clean_text(text):
    if not text:
        return text
    replacements = {
        'â€™': "'", 'â€“': "-", 'â€”': "--",
        'â€œ': '"', 'â€': '"', 'Ã©': 'é',
        'Ã¨': 'è', 'Ã': 'à', 'Ã²': 'ò',
        'Ã±': 'ñ', 'Ã¡': 'á', 'Ãº': 'ú',
        'Ã£': 'ã', 'Ã§': 'ç'
    }
    for wrong, correct in replacements.items():
        text = text.replace(wrong, correct)
    text = unicodedata.normalize('NFKC', text)
    return text.strip()

def translate_text_batch(texts, target='it'):
    try:
        pattern = re.compile(r'([^\w\s\.,;:!?\'"\-_\(\)\[\]\{\}<>@#$%&*+=\/|\\~`]+)')
        all_parts = []
        indices = []

        for text_idx, text in enumerate(texts):
            cleaned_text = clean_text(text)
            parts = pattern.split(cleaned_text or '')
            for i in range(0, len(parts), 2):
                if parts[i]:
                    all_parts.append(parts[i])
                    indices.append((text_idx, i))

        if not all_parts:
            return texts

        # ✅ Traduzione con DeepL
        translated_parts = []
        for part in all_parts:
            translated = translator_deepl.translate_text(part, source_lang="EN", target_lang=target.upper()).text
            translated_parts.append(clean_text(translated))

        result_texts = [list(pattern.split(clean_text(text) or '')) for text in texts]

        for (text_idx, part_idx), translated in zip(indices, translated_parts):
            if translated:
                result_texts[text_idx][part_idx] = translated

        return [''.join(parts) for parts in result_texts]

    except Exception as e:
        print(f"[!] Error in batch translation: {e}")
        return [clean_text(t) for t in texts]

def translate_html_batch(soup, target_language='it'):
    texts_to_translate = []
    nodes_to_replace = []

    for desc in soup.descendants:
        if isinstance(desc, NavigableString):
            txt = clean_text(str(desc).strip())
            if txt and desc.parent.name not in ['script', 'style']:
                texts_to_translate.append(txt)
                nodes_to_replace.append(desc)

    if texts_to_translate:
        translated_texts = translate_text_batch(texts_to_translate, target_language)
        for original_node, translated in zip(nodes_to_replace, translated_texts):
            try:
                if original_node != translated:
                    original_node.replace_with(translated)
            except Exception as e:
                print(f"[!] Error replacing text node: {e}")

def extract_pdf_to_html(pdf_path, temp_html_path):
    try:
        doc = fitz.open(pdf_path)
        html_parts = []
        processed_images = set()

        html_header = """<!DOCTYPE html>
        <html>
        <head>
            <meta http-equiv="Content-Type" content="text/html; charset=utf-8">
            <style>
                body {text-align: center; font-family: Arial, sans-serif; margin: 20px;}
                img {max-width: 90%; margin: 15px 0; border: 1px solid #ddd;}
                p {margin: 10px 0; line-height: 1.4em; white-space: pre-wrap;}
            </style>
        </head>
        <body>
        """
        html_parts.append(html_header)

        for page_num in range(len(doc)):
            page = doc[page_num]
            text = clean_text(page.get_text("text"))
            if text.strip():
                html_parts.append(f"<p>{text}</p>")

            image_list = page.get_images(full=True)
            for img in image_list:
                xref = img[0]
                if xref not in processed_images:
                    try:
                        base_image = doc.extract_image(xref)
                        if base_image:
                            img_data = base_image["image"]
                            img_base64 = base64.b64encode(img_data).decode("utf-8")
                            html_parts.append(f'<img src="data:image/png;base64,{img_base64}" alt="page{page_num}_img{xref}">')
                            processed_images.add(xref)
                    except Exception as img_e:
                        print(f"[!] Error processing image (xref:{xref}) on page {page_num}: {img_e}")

        html_parts.append("</body></html>")

        with open(temp_html_path, 'w', encoding='utf-8') as f:
            f.write('\n'.join(html_parts))

        return True

    except Exception as e:
        print(f"[X] Error extracting PDF to HTML: {e}")
        return False

def html_to_pdf_weasyprint(html_path, output_pdf_path):
    try:
        HTML(html_path).write_pdf(output_pdf_path)
        return True
    except Exception as e:
        print(f"[X] Error converting HTML to PDF: {e}")
        return False

def translate_pdf(file_path, target_language='it'):
    global success_count, error_count
    temp_dir = tempfile.mkdtemp(prefix="pdftrans_")
    temp_html = os.path.join(temp_dir, "temp.html")
    temp_pdf = os.path.join(temp_dir, "temp_translated.pdf")

    try:
        if not extract_pdf_to_html(file_path, temp_html):
            raise Exception("Extraction to HTML failed")

        with open(temp_html, 'r', encoding='utf-8') as f:
            soup = BeautifulSoup(f, 'html.parser')

        translate_html_batch(soup, target_language)

        with open(temp_html, 'w', encoding='utf-8') as f:
            f.write(str(soup))

        if not html_to_pdf_weasyprint(temp_html, temp_pdf):
            raise Exception("HTML to PDF conversion failed")

        shutil.move(temp_pdf, file_path)
        update_registry(file_path)
        success_count += 1
        translated_files_list.append(file_path)
        print(f"[✓] Translated: {file_path}")
        return True

    except Exception as e:
        print(f"[X] Error with file {file_path}: {e}")
        error_count += 1
        error_files_list.append(file_path)
        return False

    finally:
        shutil.rmtree(temp_dir, ignore_errors=True)

def scan_and_translate(current_dir, parallel=False, target_language='it'):
    global skipped_count
    translated_files = load_registry()
    pdf_files = []

    for root, dirs, files in os.walk(current_dir):
        for filename in files:
            if filename.lower().endswith('.pdf'):
                file_path = os.path.join(root, filename)
                if file_path not in translated_files:
                    pdf_files.append(file_path)
                else:
                    skipped_count += 1

    total_files = len(pdf_files)
    print(f"\n📊 PDF files to translate: {total_files} (skipped {skipped_count})")

    if parallel:
        with ThreadPoolExecutor(max_workers=4) as executor:
            futures = {executor.submit(translate_pdf, path, target_language): path for path in pdf_files}
            for idx, future in enumerate(as_completed(futures), start=1):
                future.result()
                print(f"[...] {idx}/{total_files} PDFs translated")
    else:
        for idx, file_path in enumerate(pdf_files, start=1):
            translate_pdf(file_path, target_language)
            print(f"[...] {idx}/{total_files} PDFs translated")

def save_report():
    try:
        with open(log_file_path, 'a', encoding='utf-8') as f:
            f.write(f"\n--- Report generated on {datetime.now().strftime('%Y-%m-%d %H:%M:%S')} ---\n")
            f.write(f"Total translated files: {success_count}\n")
            for fpath in translated_files_list:
                f.write(f"  ✔ {fpath}\n")
            f.write(f"Total error files: {error_count}\n")
            for fpath in error_files_list:
                f.write(f"  ✘ {fpath}\n")
            f.write(f"Total skipped files (already registered): {skipped_count}\n")
            f.write("✅ Translation completed.\n")
    except Exception as e:
        print(f"[X] Error saving report: {e}")

def main():
    print("Choose the target language (e.g., IT, ES, FR, DE...)")
    target = input("Language code: ").strip().lower()

    print("\nChoose translation mode:")
    print("1. Standard (sequential)")
    print("2. Batch only")
    print("3. Parallel + batch")
    choice = input("Enter 1, 2 or 3: ").strip()

    if choice == '1':
        scan_and_translate(base_dir, parallel=False, target_language=target)
    elif choice == '2':
        scan_and_translate(base_dir, parallel=False, target_language=target)
    elif choice == '3':
        scan_and_translate(base_dir, parallel=True, target_language=target)
    else:
        print("[X] Invalid choice. Exiting.")
        sys.exit(1)

    print("\n[✓] Translation completed.")

if __name__ == '__main__':
    try:
        main()
    except KeyboardInterrupt:
        print("\n[!] Interrupted by user.")
    finally:
        save_report()
        print("[✓] Report saved. Clean exit.")
